@@ -1,27 +1,15 @@
+import asyncio
+import functools
 from memorix_client_redis.features.api.hash_key import hash_key
 from uuid import uuid4
-from memorix_client_redis.features.api.json import (
-    from_dict,
-    from_json_to_any,
-    to_json_serializable,
-    to_json_from_json_serializable,
-)
-from typing import (
-    Any,
-    AsyncGenerator,
-    Generator,
-    Generic,
-    Type,
-    TypeVar,
-    cast,
-    Optional,
-)
+from memorix_client_redis.features.api.json import from_json, to_json, bytes_to_str
+import typing
 from ..api import Api, ApiDefaults
 from .task_options import TaskDequequeOptions
 
-KT = TypeVar("KT")
-PT = TypeVar("PT")
-RT = TypeVar("RT")
+KT = typing.TypeVar("KT")
+PT = typing.TypeVar("PT")
+RT = typing.TypeVar("RT")
 
 
 class TaskItemQueue(object):
@@ -29,12 +17,12 @@ class TaskItemQueue(object):
         self.queue_size = queue_size
 
 
-class TaskItemQueueWithReturns(TaskItemQueue, Generic[RT]):
+class TaskItemQueueWithReturns(TaskItemQueue, typing.Generic[RT]):
     def __init__(
         self,
         queue_size: int,
         returns_id: str,
-        returns_task: Any,
+        returns_task: typing.Any,
     ) -> None:
         self.queue_size = queue_size
         self._returns_id = returns_id
@@ -45,17 +33,17 @@ class TaskItemQueueWithReturns(TaskItemQueue, Generic[RT]):
         for message in self._returns_task.dequeue(key=self._returns_id):
             payload = message.payload
             break
-        return cast(RT, payload)
+        return typing.cast(RT, payload)
 
     async def async_get_returns(self) -> RT:
         payload = None
         async for message in self._returns_task.async_dequeue(key=self._returns_id):
             payload = message.payload
             break
-        return cast(RT, payload)
+        return typing.cast(RT, payload)
 
 
-class TaskItemDequeue(Generic[PT]):
+class TaskItemDequeue(typing.Generic[PT]):
     def __init__(
         self,
         payload: PT,
@@ -68,32 +56,33 @@ class TaskItemDequeueWithReturns(TaskItemDequeue[PT]):
         self,
         payload: PT,
         returns_id: str,
-        returns_task: Any,
+        returns_task: typing.Any,
     ) -> None:
         super().__init__(payload=payload)
         self._returns_id = returns_id
         self._returns_task = returns_task
 
     def send_returns(self, returns: RT) -> TaskItemQueue:
-        return cast(
+        return typing.cast(
             TaskItemQueue,
             self._returns_task.queue(key=self._returns_id, payload=returns),
         )
 
     async def async_send_returns(self, returns: RT) -> TaskItemQueue:
-        return cast(
+        return typing.cast(
             TaskItemQueue,
             self._returns_task.async_queue(key=self._returns_id, payload=returns),
         )
 
 
-class TaskItem(Generic[KT, PT, RT]):
+class TaskItem(typing.Generic[KT, PT, RT]):
     def __init__(
         self,
         api: Api,
         id: str,
-        payload_class: Type[PT],
-        returns_class: Optional[Type[RT]] = None,
+        payload_class: typing.Type[PT],
+        returns_class: typing.Optional[typing.Type[RT]] = None,
+        options: typing.Optional[TaskDequequeOptions] = None,
     ) -> None:
         self._api = api
         self._id = id
@@ -104,18 +93,18 @@ class TaskItem(Generic[KT, PT, RT]):
                 id="{0}_returns".format(id),
                 payload_class=returns_class,
             )
+        self._options = options
 
     def queue(self, key: KT, payload: PT) -> TaskItemQueueWithReturns[RT]:
         returns_id: str | None = None
-        payload_serializeable = to_json_serializable(value=payload)
         if hasattr(self, "_returns_task"):
             returns_id = str(uuid4())
-            wrapped_payload_json = to_json_from_json_serializable(
-                [returns_id, payload_serializeable],
+            wrapped_payload_json = to_json(
+                [returns_id, payload],
             )
         else:
-            wrapped_payload_json = to_json_from_json_serializable(
-                [payload_serializeable],
+            wrapped_payload_json = to_json(
+                [payload],
             )
 
         queue_size = self._api._redis.rpush(
@@ -123,7 +112,7 @@ class TaskItem(Generic[KT, PT, RT]):
             wrapped_payload_json,
         )
         if returns_id is None:
-            return cast(
+            return typing.cast(
                 TaskItemQueueWithReturns[RT],
                 TaskItemQueue(queue_size=queue_size),
             )
@@ -135,27 +124,28 @@ class TaskItem(Generic[KT, PT, RT]):
 
     async def async_queue(self, key: KT, payload: PT) -> TaskItemQueueWithReturns[RT]:
         print("queue async")
-        return cast(TaskItemQueueWithReturns[RT], None)
+        return typing.cast(TaskItemQueueWithReturns[RT], None)
 
     def dequeue(
         self,
         key: KT,
-        options: Optional[TaskDequequeOptions] = None,
-    ) -> Generator[TaskItemDequeueWithReturns[PT], None, None]:
-        take_newest: Optional[bool] = None
+        options: typing.Optional[TaskDequequeOptions] = None,
+    ) -> typing.Generator[TaskItemDequeueWithReturns[PT], None, None]:
+        merged_options = self._options
         try:
-            take_newest = cast(TaskDequequeOptions, options).take_newest
+            merged_options = TaskDequequeOptions.merge(
+                typing.cast(ApiDefaults, self._api._defaults).task_dequeque_options,
+                self._options,
+            )
         except AttributeError:
-            try:
-                take_newest = cast(
-                    TaskDequequeOptions,
-                    cast(ApiDefaults, self._api._defaults).task_dequeque_options,
-                ).take_newest
-            except AttributeError:
-                pass
+            pass
+        merged_options = TaskDequequeOptions.merge(
+            merged_options,
+            options,
+        )
 
         while True:
-            if take_newest:
+            if merged_options is not None and merged_options.take_newest:
                 [channel_bytes, data_bytes] = self._api._redis.brpop(
                     hash_key(self._id, key=key),
                 )
@@ -164,21 +154,26 @@ class TaskItem(Generic[KT, PT, RT]):
                     hash_key(self._id, key=key),
                 )
 
-            wrapped_payload = from_json_to_any(data_bytes)
+            data_str = bytes_to_str(data_bytes)
+
             if hasattr(self, "_returns_task"):
-                returns_id = wrapped_payload[0]
-                payload_dict = wrapped_payload[1]
-                payload = from_dict(dict=payload_dict, data_class=self._payload_class)
+                seperator_index = data_str.index(",")
+                returns_id = data_str[2 : seperator_index - 1]
+                payload_str = data_str[seperator_index + 1 : -1]
+                payload = from_json(
+                    payload_str,
+                    self._payload_class,
+                )
                 yield TaskItemDequeueWithReturns(
                     payload=payload,
                     returns_id=returns_id,
                     returns_task=self._returns_task,
                 )
             else:
-                payload_dict = wrapped_payload[0]
-                payload = from_dict(dict=payload_dict, data_class=self._payload_class)
+                payload_str = data_str[1:-1]
+                payload = from_json(payload_str, self._payload_class)
 
-                yield cast(
+                yield typing.cast(
                     TaskItemDequeueWithReturns[PT],
                     TaskItemDequeue(payload=payload),
                 )
@@ -186,9 +181,25 @@ class TaskItem(Generic[KT, PT, RT]):
     async def async_dequeue(
         self,
         key: KT,
-    ) -> AsyncGenerator[TaskItemDequeueWithReturns[PT], None]:
+    ) -> typing.AsyncGenerator[TaskItemDequeueWithReturns[PT], None]:
         print("dequeue async")
-        yield cast(TaskItemDequeueWithReturns[PT], None)
+        yield typing.cast(TaskItemDequeueWithReturns[PT], None)
+
+    def clear(self, key: KT) -> None:
+        self._api._redis.delete(
+            hash_key(self._id, key=key),
+        )
+
+    async def async_clear(self, key: KT) -> None:
+        loop = asyncio.get_running_loop()
+        await loop.run_in_executor(
+            None,
+            functools.partial(
+                TaskItem.clear,
+                self=self,
+                key=key,
+            ),
+        )
 
 
 class TaskItemNoKey(TaskItem[None, PT, RT]):
@@ -201,20 +212,28 @@ class TaskItemNoKey(TaskItem[None, PT, RT]):
         return await TaskItem.async_queue(self, key=None, payload=payload)
 
     # Different signature on purpose
-    def dequeue(self) -> Generator[TaskItemDequeueWithReturns[PT], None, None]:  # type: ignore
+    def dequeue(self) -> typing.Generator[TaskItemDequeueWithReturns[PT], None, None]:  # type: ignore
         return TaskItem.dequeue(self, key=None)
 
     # Different signature on purpose
-    async def async_dequeue(self) -> AsyncGenerator[TaskItemDequeueWithReturns[PT], None]:  # type: ignore
+    async def async_dequeue(self) -> typing.AsyncGenerator[TaskItemDequeueWithReturns[PT], None]:  # type: ignore
         return TaskItem.async_dequeue(self, key=None)
 
+    # Different signature on purpose
+    def clear(self) -> None:  # type: ignore
+        TaskItem.clear(self, key=None)
+
+    # Different signature on purpose
+    async def async_clear(self) -> None:  # type: ignore
+        await TaskItem.async_clear(self, key=None)
+
 
 class TaskItemNoReturns(TaskItem[KT, PT, None]):
     def __init__(
         self,
         api: Api,
         id: str,
-        payload_class: Type[PT],
+        payload_class: typing.Type[PT],
     ) -> None:
         super().__init__(
             api=api,
@@ -225,34 +244,56 @@ class TaskItemNoReturns(TaskItem[KT, PT, None]):
 
     # Different signature on purpose
     def queue(self, key: KT, payload: PT) -> TaskItemQueue:  # type: ignore
-        return cast(TaskItemQueue, TaskItem.queue(self, key=key, payload=payload))
+        return typing.cast(
+            TaskItemQueue,
+            TaskItem.queue(self, key=key, payload=payload),
+        )
 
     # Different signature on purpose
     async def async_queue(self, key: KT, payload: PT) -> TaskItemQueue:  # type: ignore
         return await TaskItem.async_queue(self, key=key, payload=payload)
 
     # Different signature on purpose
-    def dequeue(self, key: KT) -> Generator[TaskItemDequeue[PT], None, None]:  # type: ignore
+    def dequeue(self, key: KT) -> typing.Generator[TaskItemDequeue[PT], None, None]:  # type: ignore
         return TaskItem.dequeue(self, key=key)
 
     # Different signature on purpose
-    async def async_dequeue(self, key: KT) -> AsyncGenerator[TaskItemDequeue[PT], None]:  # type: ignore
+    async def async_dequeue(self, key: KT) -> typing.AsyncGenerator[TaskItemDequeue[PT], None]:  # type: ignore
         return TaskItem.async_dequeue(self, key=key)
 
+    # Different signature on purpose
+    def clear(self, key: KT) -> None:
+        TaskItem.clear(self, key=key)
+
+    # Different signature on purpose
+    async def async_clear(self, key: KT) -> None:
+        await TaskItem.async_clear(self, key=key)
+
 
 class TaskItemNoKeyNoReturns(TaskItemNoReturns[None, PT]):
     # Different signature on purpose
     def queue(self, payload: PT) -> TaskItemQueue:  # type: ignore
-        return cast(TaskItemQueue, TaskItem.queue(self, key=None, payload=payload))
+        return typing.cast(
+            TaskItemQueue,
+            TaskItem.queue(self, key=None, payload=payload),
+        )
 
     # Different signature on purpose
     async def async_queue(self, payload: PT) -> TaskItemQueue:  # type: ignore
         return await TaskItem.async_queue(self, key=None, payload=payload)
 
     # Different signature on purpose
-    def dequeue(self) -> Generator[TaskItemDequeue[PT], None, None]:  # type: ignore
+    def dequeue(self) -> typing.Generator[TaskItemDequeue[PT], None, None]:  # type: ignore
         return TaskItem.dequeue(self, key=None)
 
     # Different signature on purpose
-    async def async_dequeue(self) -> AsyncGenerator[TaskItemDequeue[PT], None]:  # type: ignore
+    async def async_dequeue(self) -> typing.AsyncGenerator[TaskItemDequeue[PT], None]:  # type: ignore
         return TaskItem.async_dequeue(self, key=None)
+
+    # Different signature on purpose
+    def clear(self) -> None:  # type: ignore
+        TaskItem.clear(self, key=None)
+
+    # Different signature on purpose
+    async def async_clear(self) -> None:  # type: ignore
+        await TaskItem.async_clear(self, key=None)
